/*
*  Name:        defs.hh
*  Author:      Rafael Jesus Alcantara Perez
*  Contributor: Francisco Vides Fernndez
*  Summary:     UESQLC common definitions
*  Date:        $Date: 2001/02/06 09:12:38 $
*  Revision:    $Revision: 1.13 $
*
*  Copyright (C) 1996-1999  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _DEFS_UESQLC__
#define _DEFS_UESQLC__

#include "exceptions.hh"


///  Value for YACC stack.
#define YYSTYPE  TStackValue


///  Defines SQL-92 Conformance Levels.
enum ESQL92_ConformanceLevels
{
  eSTRICT_SQL_92 = 1      , ///<  Strict SQL 92 (not relaxed).
  eENTRY_SQL_LEVEL        , ///<  Entry SQL level.
  eINTERMEDIATE_SQL_LEVEL , ///<  Intermediate SQL level.
  eFULL_SQL_LEVEL           ///<  Full SQL level.
};


/**
*  Defines identifiers for SQL-92 Rules. The  meaning of the enumerated values
*  is,  first  there  is  the prefix SQL92, second F, SR or LR (Format, Syntax
*  Rule or Leveling Rule), next are the  section, and  lastly the point number
*  (if any).
*
*  Identifiers come from document X3H2-92-154/DBL CBR-002:
*
*    (Second Informal Review Draft) ISO/IEC 9075:1992, Database
*    Language SQL- July 30, 1992
*/
enum ESQL92_Requirements 
{
  eSQL92_F_5_4_A = 1  ,
  eSQL92_F_5_4_B      ,
  eSQL92_F_5_4_C      ,
  eSQL92_F_19_1_A     ,
  eSQL92_F_19_2_A     ,
  eSQL92_SR_5_2_P8    ,
  eSQL92_SR_5_2_P9    ,
  eSQL92_SR_5_3_P21   ,
  eSQL92_SR_8_2_P1    ,
  eSQL92_SR_11_3_P5   ,
  eSQL92_SR_11_19_P10 ,
  eSQL92_SR_13_5_P1   ,
  eSQL92_SR_13_8_P5   ,
  eSQL92_SR_13_11_P4  ,
  eSQL92_LR_5_2_P1    ,
  eSQL92_LR_5_2_P2_A  ,
  eSQL92_LR_5_2_P2_B  ,
  eSQL92_LR_6_3_P2_B  ,
  eSQL92_LR_6_3_P2_C  ,
  eSQL92_LR_6_8_P1    ,
  eSQL92_LR_6_8_P2    ,
  eSQL92_LR_6_11_P2_A ,
  eSQL92_LR_6_11_P2_B ,
  eSQL92_LR_6_11_P2_C ,
  eSQL92_LR_6_11_P2_D ,
  eSQL92_LR_8_9_P2    ,
  eSQL92_LR_8_10_P1   ,
  eSQL92_LR_11_4_P2_A ,
  eSQL92_LR_11_10_P2  ,
  eSQL92_LR_13_1_P1_A ,
  eSQL92_LR_13_1_P2_A ,
  eSQL92_LR_13_1_P2_B ,
  eSQL92_LR_13_3_P2_A ,
  eSQL92_LR_13_3_P2_B
};


#endif  // not _DEFS_UESQLC__
