/*
*  Name:      exceptions.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC exceptions
*  Date:      $Date: 2002/09/13 23:50:33 $
*  Revision:  $Revision: 1.16 $
*
*  Copyright (C) 1996-2001  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _EXCEPTIONS_UESQLC__
#define _EXCEPTIONS_UESQLC__

#include <ostream>
#include <mpcl/base_exception.hh>
#include <mpcl/error_exception.hh>
#include <mpcl/fatal_exception.hh>
#include <mpcl/warning_exception.hh>


/**
*  Exception: parent exception class for UESQLC.
*
*  Warning and errors should look like this:
*
*    source-file-name:lineno: message
*/
class TException: public virtual mpcl::TBaseException 
{

  protected:

    //
    //  S E L E C T O R S
    //

    void write (std::basic_ostream<char>& rtTARGET_OSTREAM) const;

};  // class TException


///  Exception: general warning.
class TWarningException                :
  private virtual mpcl::TBaseException ,
  public mpcl::TWarningException       ,
  public TException
{

  public:

    //
    //  C O N S T R U C T O R S
    //

    /**
    *  Builds the instance with a generic and a (if used) specific description
    *  of the exception.
    *  @param pkcSPECIFIC_DESCRIPTION Specific description.
    *  @param pkcFILE_NAME            File name where the exception has thrown.
    *  @param kiLINE_NUMBER           Line number of the former file.
    */
    TWarningException ( const char* pkcSPECIFIC_DESCRIPTION ,
                        const char* pkcFILE_NAME            ,
                        const int   kiLINE_NUMBER           );

};  // class TWarningException


///  Exception: general error.
class TErrorException                  :
  private virtual mpcl::TBaseException ,
  public mpcl::TErrorException         ,
  public TException
{

  public:

    //
    //  C O N S T R U C T O R S
    //

    /**
    *  Builds the instance with a generic and a (if used) specific description
    *  of the exception.
    *  @param pkcSPECIFIC_DESCRIPTION Specific description.
    *  @param pkcFILE_NAME            File name where the exception has thrown.
    *  @param kiLINE_NUMBER           Line number of the former file.
    */
    TErrorException ( const char* pkcSPECIFIC_DESCRIPTION ,
                      const char* pkcFILE_NAME            ,
                      const int   kiLINE_NUMBER           );

};  // class TErrorException


/**
*  Exception: compiler runtime error.
*
*  Fatal errors should look like this:
*
*    program:source-file-name:lineno: message
*/
class TFatalException                  :
  private virtual mpcl::TBaseException ,
  public mpcl::TFatalException         ,
  public TException
{

  protected:

    //
    //  S E L E C T O R S
    //

    void write (std::basic_ostream<char>& rtTARGET_OSTREAM) const;


  public:

    //
    //  C O N S T R U C T O R S
    //

    /**
    *  Builds the instance with a generic and a (if used) specific description
    *  of the exception.
    *  @param pkcSPECIFIC_DESCRIPTION Specific description.
    *  @param pkcFILE_NAME            File name where the exception has thrown.
    *  @param kiLINE_NUMBER           Line number of the former file.
    */
    TFatalException ( const char* pkcSPECIFIC_DESCRIPTION ,
                      const char* pkcFILE_NAME  = ""      ,
                      const int   kiLINE_NUMBER = 0       );

};  // class TFatalException


#endif  // not _EXCEPTIONS_UESQLC__
